package cn.iocoder.yudao.module.sale.controller.admin.deptTarget;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.datapermission.core.annotation.DataPermission;
import cn.iocoder.yudao.framework.datapermission.core.dept.rule.DeptDataPermissionRule;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.order.service.targetLog.TargetLogService;
import cn.iocoder.yudao.module.sale.convert.deptTarget.DeptTargetConvert;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.DeptTargetDO;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.PersonalDeptTargetDO;
import cn.iocoder.yudao.module.sale.dto.AchievementStatisticsDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementRetDto;
import cn.iocoder.yudao.module.sale.service.deptTarget.DeptTargetService;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetBackVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetQueryVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetUpdateReqVO;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.*;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 部门业绩目标设置")
@RequestMapping("/sale/dept-target")
public class DeptTargetController {

    @Resource
    private DeptTargetService deptTargetService;

    @Resource
    private TargetLogService targetLogService;

    @Resource
    private FileMakeApi fileMakeApi;

    @PostMapping("/create")
    @ApiOperation("创建部门业绩目标设置")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:create')")
    public CommonResult<Long> createDeptTarget(@Valid @RequestBody DeptTargetCreateReqVO createReqVO) {
        return success(deptTargetService.createDeptTarget(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新部门业绩目标设置")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:update')")
    public CommonResult<Boolean> updateDeptTarget(@Valid @RequestBody DeptTargetUpdateReqVO updateReqVO) {
        deptTargetService.updateDeptTarget(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除部门业绩目标设置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:delete')")
    public CommonResult<Boolean> deleteDeptTarget(@RequestParam("id") Long id) {
        deptTargetService.deleteDeptTarget(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得部门业绩目标设置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<DeptTargetBackVO> getDeptTarget(@RequestParam("id") Long id) {
        DeptTargetDO deptTarget = deptTargetService.getDeptTarget(id);
        return success(DeptTargetConvert.INSTANCE.convert(deptTarget));
    }

    @GetMapping("/list")
    @ApiOperation("获得部门业绩目标设置列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<List<DeptTargetBackVO>> getDeptTargetList(@RequestParam("ids") Collection<Long> ids) {
        List<DeptTargetDO> list = deptTargetService.getDeptTargetList(ids);
        return success(DeptTargetConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得部门业绩目标设置分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<PageResult<DeptTargetBackVO>> getDeptTargetPage(@Valid DeptTargetQueryVO query, PageVO page) {
        PageResult<DeptTargetDO> pageResult = deptTargetService.getDeptTargetPage(query, page);
        return success(DeptTargetConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("personal/page")
    @ApiOperation("获得个人目标进度分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<PageResult<PersonalDeptTargetDO>> getPersonalDeptTargetPage(@Valid DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        PageResult<PersonalDeptTargetDO> pageResult = deptTargetService.getPersonalDeptTargetPage(query, pageVO);
        return success(pageResult);
    }

    @GetMapping("statistics/page")
    @ApiOperation("获得部门业绩目标统计分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<PageResult<DeptTargetBackVO>> getDeptTargetStatisticsPage(DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        PageResult<DeptTargetBackVO> pageResult = deptTargetService.getDeptTargetStatisticsPage(query, pageVO);
        return success(pageResult);
    }

    @GetMapping("deptStatistics/page")
    @ApiOperation("获得我的部门及以下部门业绩目标统计分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<PageResult<DeptTargetBackVO>> getMyDeptTargetStatisticsPage(DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        query.setFlag("my");
        PageResult<DeptTargetBackVO> pageResult = deptTargetService.getDeptTargetStatisticsPage(query, pageVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出部门业绩目标设置 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:export')")
    @OperateLog(type = EXPORT)
    public void exportDeptTargetExcel(@Valid DeptTargetQueryVO query,
              HttpServletResponse response) throws IOException {
        List<DeptTargetDO> list = deptTargetService.getDeptTargetList(query);
        // 导出 Excel
        List<DeptTargetBackVO> datas = DeptTargetConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "部门业绩目标设置.xls", "数据", DeptTargetBackVO.class, datas);
    }

    @GetMapping("/getCreateInitData")
    @ApiOperation("获取设置部门业绩初始化数据")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<Map<String,Object>> getCreateInitData(){
        Map<String,Object> map = deptTargetService.getCreateInitData();
        return success(map);
    }

    @PostMapping("/myAchievementByPage")
    @ApiOperation("获得我的业绩分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<MyAchievementRetDto> myAchievementByPage(@RequestBody DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        query.setFlag("my");
        MyAchievementRetDto pageResult = deptTargetService.myAchievementByPage(query, pageVO);
        return success(pageResult);
    }

    @PostMapping("/myDeptAchievementByPage")
    @ApiOperation("获得我的部门业绩分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<MyAchievementRetDto> myDeptAchievementByPage(@RequestBody DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        MyAchievementRetDto pageResult = deptTargetService.myDeptAchievementByPage(query, pageVO);
        return success(pageResult);
    }

    @PostMapping("/allAchievementByPage")
    @DataPermission(includeRules = {DeptDataPermissionRule.class})
    @ApiOperation("全部业绩分页")
    //@PreAuthorize("@ss.hasPermission('ecw:dept-target:query')")
    public CommonResult<MyAchievementRetDto> allAchievementByPage(@RequestBody DeptTargetQueryVO query) {
        PageVO pageVO = new PageVO();
        pageVO.setPage(query.getPageNo());
        pageVO.setRows(query.getPageSize());
        MyAchievementRetDto pageResult = deptTargetService.myAchievementByPage(query, pageVO);
        return success(pageResult);
    }

    @PutMapping("/compensateTargetLog")
    @ApiOperation(value = "补偿业绩日志")
    @ApiImplicitParam(value = "订单编号", name = "orderNumbers", required = true, example = "示例：NG2300208L,NG2300212L,NG2300209L多个订单编号逗号连接", dataTypeClass = String.class)
    public void testTargetLog(@RequestParam(value = "orderNumbers")String orderNumbers) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(orderNumbers)){
            list.addAll(Arrays.asList(orderNumbers.split(StrUtil.COMMA)));
        }
        targetLogService.compensateTargetLog(list);
    }


    @GetMapping("/exportDetailExcel")
    @ApiOperation("导出业绩明细")
    @PreAuthorize("@ss.hasPermission('ecw:achievement:export')")
    @OperateLog(type = EXPORT)
    public void exportDetailExcel(@Valid DeptTargetQueryVO exportReqVO,
                                  HttpServletResponse response) throws IOException {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(DownloadTypeEnum.ACHIEVEMENT_DETAIL_EXCEL_EXPORT.getType());
        reqDTO.setName("业绩明细");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSON.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
//        List<MyAchievementExportDto> list = DeptTargetConvert.INSTANCE.convertExportList(deptTargetService.getExportDetailExcel(exportReqVO));
//        ExcelUtils.write(response, "业绩明细.xls", "数据", MyAchievementExportDto.class, list);
    }

    @GetMapping("/exportStatisticsExcel")
    @ApiOperation("导出业绩统计")
    @PreAuthorize("@ss.hasPermission('ecw:achievement:export')")
    @OperateLog(type = EXPORT)
    public void exportStatisticsExcel(@Valid DeptTargetQueryVO exportReqVO,
                                    HttpServletResponse response) throws IOException {
        //TODO
        List<AchievementStatisticsDto> list = deptTargetService.getExportStatisticsExcel(exportReqVO);
//        List<CurrencyDO> list = currencyService.getCurrencyList(exportReqVO);
//        // 导出 Excel
//        List<CurrencyExcelVO> datas = CurrencyConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "业绩统计.xls", "数据", AchievementStatisticsDto.class, list);
    }
}
